@@ -28,6 +28,7 @@
 *    Ellad B. Tadmor
 *    Valeriu Smirichinski
 *    Stephen M. Whalen
+*    Yaser Afshar
 *
 */
 
@@ -98,6 +99,7 @@ static void calc_phi(double * epsilon,
                      double * cutoff,
                      double r,
                      double * phi);
+
 static void calc_phi_dphi(double * epsilon,
                           double * C,
                           double * Rzero,
@@ -141,6 +143,13 @@ static void calc_phi(double * epsilon,
                      double r,
                      double * phi)
 {
+  if (r > *cutoff)
+  {
+    /* Argument exceeds cutoff radius */
+    *phi = 0.0;
+    return;
+  }
+
   /* local variables */
   double ep;
   double ep2;
@@ -148,15 +157,7 @@ static void calc_phi(double * epsilon,
   ep = exp(-(*C) * (r - *Rzero));
   ep2 = ep * ep;
 
-  if (r > *cutoff)
-  {
-    /* Argument exceeds cutoff radius */
-    *phi = 0.0;
-  }
-  else
-  {
-    *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
-  }
+  *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
 
   return;
 }
@@ -171,6 +172,14 @@ static void calc_phi_dphi(double * epsilon,
                           double * phi,
                           double * dphi)
 {
+  if (r > *cutoff)
+  {
+    /* Argument exceeds cutoff radius */
+    *phi = 0.0;
+    *dphi = 0.0;
+    return;
+  }
+
   /* local variables */
   double ep;
   double ep2;
@@ -178,17 +187,8 @@ static void calc_phi_dphi(double * epsilon,
   ep = exp(-(*C) * (r - *Rzero));
   ep2 = ep * ep;
 
-  if (r > *cutoff)
-  {
-    /* Argument exceeds cutoff radius */
-    *phi = 0.0;
-    *dphi = 0.0;
-  }
-  else
-  {
-    *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
-    *dphi = 2.0 * (*epsilon) * (*C) * (-ep + ep2);
-  }
+  *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
+  *dphi = 2.0 * (*epsilon) * (*C) * (-ep + ep2);
 
   return;
 }
@@ -207,6 +207,15 @@ static void calc_phi_d2phi(double * epsilon,
                            double * dphi,
                            double * d2phi)
 {
+  if (r > *cutoff)
+  {
+    /* Argument exceeds cutoff radius */
+    *phi = 0.0;
+    *dphi = 0.0;
+    *d2phi = 0.0;
+    return;
+  }
+
   /* local variables */
   double ep;
   double ep2;
@@ -214,19 +223,9 @@ static void calc_phi_d2phi(double * epsilon,
   ep = exp(-(*C) * (r - *Rzero));
   ep2 = ep * ep;
 
-  if (r > *cutoff)
-  {
-    /* Argument exceeds cutoff radius */
-    *phi = 0.0;
-    *dphi = 0.0;
-    *d2phi = 0.0;
-  }
-  else
-  {
-    *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
-    *dphi = 2.0 * (*epsilon) * (*C) * (-ep + ep2);
-    *d2phi = 2.0 * (*epsilon) * (*C) * (*C) * (ep - 2.0 * ep2);
-  }
+  *phi = (*epsilon) * (-ep2 + 2.0 * ep) + *shift;
+  *dphi = 2.0 * (*epsilon) * (*C) * (-ep + ep2);
+  *d2phi = 2.0 * (*epsilon) * (*C) * (*C) * (ep - 2.0 * ep2);
 
   return;
 }
@@ -257,6 +256,8 @@ compute(KIM_ModelCompute const * const modelCompute,
   double * pRij = &(Rij[0]);
   double Rij_pairs[2][3];
   double * pRij_pairs = &(Rij_pairs[0][0]);
+  double v;
+  double vir[6];
   int ier;
   int i;
   int i_pairs[2];
@@ -271,6 +272,8 @@ compute(KIM_ModelCompute const * const modelCompute,
   int comp_energy;
   int comp_force;
   int comp_particleEnergy;
+  int comp_virial;
+  int comp_particleVirial;
   int comp_process_dEdr;
   int comp_process_d2Edr2;
 
@@ -287,6 +290,8 @@ compute(KIM_ModelCompute const * const modelCompute,
   double * energy;
   double * force;
   double * particleEnergy;
+  double * virial;
+  double * particleVirial;
   int numOfAtomNeigh;
 
   /* Get model buffer */
@@ -340,8 +345,15 @@ compute(KIM_ModelCompute const * const modelCompute,
         || KIM_ModelComputeArguments_GetArgumentPointerDouble(
                modelComputeArguments,
                KIM_COMPUTE_ARGUMENT_NAME_partialParticleEnergy,
-               &particleEnergy);
-
+               &particleEnergy)
+        || KIM_ModelComputeArguments_GetArgumentPointerDouble(
+               modelComputeArguments,
+               KIM_COMPUTE_ARGUMENT_NAME_partialVirial,
+               &virial)
+        || KIM_ModelComputeArguments_GetArgumentPointerDouble(
+               modelComputeArguments,
+               KIM_COMPUTE_ARGUMENT_NAME_partialParticleVirial,
+               &particleVirial);
   if (ier == TRUE)
   {
     LOG_ERROR("Could not retrieve compute arguments");
@@ -352,6 +364,8 @@ compute(KIM_ModelCompute const * const modelCompute,
   comp_energy = (energy != 0);
   comp_force = (force != 0);
   comp_particleEnergy = (particleEnergy != 0);
+  comp_virial = (virial != 0);
+  comp_particleVirial = (particleVirial != 0);
 
   /* Check to be sure that the species are correct */
   ier = TRUE;
@@ -379,6 +393,18 @@ compute(KIM_ModelCompute const * const modelCompute,
     }
   }
 
+  if (comp_virial)
+  {
+    for (i = 0; i < 6; ++i) virial[i] = 0.0;
+  }
+
+  if (comp_particleVirial)
+  {
+    for (i = 0; i < *nAtoms; ++i)
+    {
+      for (k = 0; k < 6; ++k) particleVirial[i * 6 + k] = 0.0;
+    }
+  }
 
   /* Compute energy and forces */
 
@@ -401,7 +427,6 @@ compute(KIM_ModelCompute const * const modelCompute,
         return ier;
       }
 
-
       /* loop over the neighbors of atom i */
       for (jj = 0; jj < numOfAtomNeigh; ++jj)
       {
@@ -443,7 +468,8 @@ compute(KIM_ModelCompute const * const modelCompute,
                 d2Eidr = 0.5 * d2phi;
               }
             }
-            else if (comp_force || comp_process_dEdr)
+            else if (comp_force || comp_process_dEdr ||
+                     comp_virial || comp_particleVirial)
             {
               /* compute pair potential and its derivative */
               calc_phi_dphi(epsilon, C, Rzero, shift, cutoff, R, &phi, &dphi);
@@ -480,7 +506,57 @@ compute(KIM_ModelCompute const * const modelCompute,
             if (comp_process_dEdr)
             {
               ier = KIM_ModelComputeArguments_ProcessDEDrTerm(
-                  modelComputeArguments, dEidr, R, pRij, i, j);
+                modelComputeArguments, dEidr, R, pRij, i, j);
+            }
+            /* contribution to virial and particle virial */
+            if (comp_virial || comp_particleVirial)
+            {
+              /* Virial has 6 components and is stored as a 6-element
+                 vector in the following order: xx, yy, zz, yz, xz, xy. */
+              v = dEidr / R;
+
+              vir[0] = v * Rij[0] * Rij[0];
+              vir[1] = v * Rij[1] * Rij[1];
+              vir[2] = v * Rij[2] * Rij[2];
+              vir[3] = v * Rij[1] * Rij[2];
+              vir[4] = v * Rij[0] * Rij[2];
+              vir[5] = v * Rij[0] * Rij[1];
+
+              if (comp_virial)
+              {
+                virial[0] += vir[0];
+                virial[1] += vir[1];
+                virial[2] += vir[2];
+                virial[3] += vir[3];
+                virial[4] += vir[4];
+                virial[5] += vir[5];
+              }
+
+              if (comp_particleVirial)
+              {
+                vir[0] *= 0.5;
+                vir[1] *= 0.5;
+                vir[2] *= 0.5;
+                vir[3] *= 0.5;
+                vir[4] *= 0.5;
+                vir[5] *= 0.5;
+
+                k = i * 6;
+                particleVirial[k + 0] += vir[0];
+                particleVirial[k + 1] += vir[1];
+                particleVirial[k + 2] += vir[2];
+                particleVirial[k + 3] += vir[3];
+                particleVirial[k + 4] += vir[4];
+                particleVirial[k + 5] += vir[5];
+
+                k = j * 6;
+                particleVirial[k + 0] += vir[0];
+                particleVirial[k + 1] += vir[1];
+                particleVirial[k + 2] += vir[2];
+                particleVirial[k + 3] += vir[3];
+                particleVirial[k + 4] += vir[4];
+                particleVirial[k + 5] += vir[5];
+              }
             }
 
             /* contribution to process_d2Edr2 */
@@ -948,6 +1024,18 @@ static int compute_arguments_create(
                KIM_COMPUTE_ARGUMENT_NAME_partialForces,
                KIM_SUPPORT_STATUS_optional);
 
+  ier = ier
+        || KIM_ModelComputeArgumentsCreate_SetArgumentSupportStatus(
+               modelComputeArgumentsCreate,
+               KIM_COMPUTE_ARGUMENT_NAME_partialVirial,
+               KIM_SUPPORT_STATUS_optional);
+
+  ier = ier
+        || KIM_ModelComputeArgumentsCreate_SetArgumentSupportStatus(
+               modelComputeArgumentsCreate,
+               KIM_COMPUTE_ARGUMENT_NAME_partialParticleVirial,
+               KIM_SUPPORT_STATUS_optional);
+
   /* register call backs */
   LOG_INFORMATION("Register call back supportStatus");
   ier = ier
