@@ -592,7 +592,7 @@ recursive subroutine Compute_Energy_Forces(model_compute_handle, &
 
   !-- Local variables
   real(c_double) :: Rij(DIM)
-  real(c_double) :: r,Rsqij,phi,dphi,g,dg,dU,U,dphieff = 0.0_cd
+  real(c_double) :: r,Rsqij,phi,dphi,g,dg,dU,U,dphieff
   real(c_double) :: dphii,dUi,dphij,dUj
   integer(c_int) :: i,j,jj,numnei,comp_force,comp_particleEnergy,comp_virial,comp_energy
   integer(c_int) :: ierr2
@@ -705,6 +705,7 @@ recursive subroutine Compute_Energy_Forces(model_compute_handle, &
   if (comp_energy.eq.1) energy = 0.0_cd
   if (comp_force.eq.1) force  = 0.0_cd
   if (comp_virial.eq.1) virial = 0.0_cd
+  dphieff = 0.0_cd
 
   allocate( rho(N) )  ! pair functional electron density
   rho = 0.0_cd
